# Brain Art Demo

## Overview
This folder powers brain-art generation using the IDUN Guardian and Touchdesigner software.
The demo includes 2 components: 

**Real-time EEG Predictions**: The demo script records EEG data and processes it in real time to extract key features for live monitoring:
- **Filtered EEG Data**: Provides a clearer view of the EEG signal.
- **Frequency Bands**: Calculates Power Spectrum Density (PSD) and FFT band z-scores for frequency-based analysis.
- **IMU Data**: Provides information about head movement (accelerometer, magnetometer and gyroscope)
- **Jaw Clench and Eye Movement Detection**: Detects jaw clenches and horizontal eye movements (left or right) using two classifiers.

**BrainArt with TouchDesigner**: Real-time predictions are transmitted via UDP to TouchDesigner, creating a BrainArt demo that visually represents brain activity. FFT z-scores values for each band control color, size, speed of the particles. Eye movements can bring the particle cloud left and right. Jaw clenches create a surge in the particles. 

---

## Installation

### Prerequisites
- **Python >=3.9**: Download [here](https://www.python.org/downloads/).
- **Git**: Download [here](https://git-scm.com).
- **Pipenv**: Install via `pip`:
  ```bash
  > pip install pipenv
  ```

### Installation

#### 1. Setup the environment
1. **Download the folder and open a terminal**: 

2. **Create a Virtual Environment Folder in the downloaded folder**:
   ```bash
   > mkdir .venv
   ```

3. **Install Dependencies**:
   ```bash
   > pipenv install
   ```

3. **Activate the Environment**:
   ```bash
   > pipenv shell
   ```
   You can exit the environment using 
   ```bash
   > exit
   ```


#### 2. Setup TouchDesigner
1. Download [TouchDesigner](https://derivative.ca/download) and create an account.
2. Open the provided TouchDesigner file (touchdesigner/DEMO.toe).
3. Start the animation, which will automatically visualize the BrainArt when the demo script is running.
   

### Usage

Run in the pipenv shell: 

```bash
> python brain-art-demo.py
```

This will both open the signal visualizer and send some UDP data to TouchDesigner to control the brain art. 

> **Note:** You need to wait to receive FFT data (a few minutes) before you can control the brain art. 


## Troubleshooting

- **Environment Issues**: If problems occur, delete the `.venv` folder and `Pipfile.lock`, then rerun `pipenv install`. 
